import smbus
import time
from os import system
import smtplib
import xml.etree.ElementTree as ET

# define the I2C bus device
bus=smbus.SMBus(1)

# debug
debug = 1
# device constants
PCA9536=0x41    #7 bit address (will be shifted to add the read/write bit)
PCF8574=0x20
ANALOG_SCALE = float(15.97/255)
LED1 = 0x04
LED2 = 0x08
OUT1 = 0x01
OUT2 = 0x02
MAX11600_ADDRESS = 0x64 #shifted left
MAX11600_CHANNEL0 = 0x00
MAX11600_CHANNEL1 = 0x02
MAX11600_CHANNEL2 = 0x04
MAX11600_CHANNEL3 = 0x06
MAX11600_SINGLE = 0x01
MAX11600_SCAN_SINGLE =0x60  
MAX11600_SETUP = 0x80
MAX11600_REF = 0x50
MAX11600_RST = 0x02
MAX11600_CLK = 0x08

ANALOG_type = 0
ANALOG_chan = 1
ANALOG_enabled = 2
ANALOG_name = 3
ANALOG_units = 4
ANALOG_max = 5
ANALOG_min = 6
ANALOG_hysteresis = 7
DIGITAL_type = 0
DIGITAL_chan = 1
DIGITAL_enabled = 2
DIGITAL_name = 3
DIGITAL_error = 4
DIGITAL_hysteresis = 5

# send an email message
def SendMsg(server,port, message, sender, receiver):
    message = ("From: %s\r\nTo: %s\r\n\r\n" % (sender, receiver)) + message
    if debug: print 'Email msg' + message
    try:
        server = smtplib.SMTP(server,port)
	server.starttls()
	server.login(sender,password)
	server.sendmail(sender, receiver, message)
	server.quit()
    except smtplib.SMTPAuthenticationError:
	print 'Authentication Error'
    except smtplib.SMTPConnectError:
        print 'Connetion Error'
    except smtplib.SMTPServerDisconect:
        print 'Server Disconnect'
    except smtplib.SMTPDataError:
        print 'Data Error'
    except smtplib.Exception:
        print 'Email Exception'

# Initialize the Analog to Digital Converter
def MAX11600_init():
    out = MAX11600_SETUP+MAX11600_REF+MAX11600_RST+MAX11600_CLK
    if debug: print "MAX11600 Setup:",hex(out)
    bus.write_byte(MAX11600_ADDRESS,out)

# Read the Analog to Digital Converter and apply the scale factor
# Returns a float from 0 - 15V
def MAX11600_read(channel):
    out = MAX11600_SCAN_SINGLE+channel+MAX11600_SINGLE
#   print "MAX11600-Conf:",hex(out)
    bus.write_byte(MAX11600_ADDRESS,out)
    return ANALOG_SCALE*bus.read_byte(MAX11600_ADDRESS)
    
# Initialize the Digital Output device
def PCA9536_init():
    # set register 3 to all outputs
    bus.write_byte_data(PCA9536,3,0x00)
    #turn off all outputs
    bus.write_byte_data(PCA9536,1,0xFF)

# Write a value to the output
def PCA9536_output(out):
    # flip the bits
    out = (0xFF ^ out) & 0x0F
    #print "out:",out
    bus.write_byte_data(PCA9536,1,out)
 
# Read a value from the Digital Input Device
def PCF8574_input():
    val =  bus.read_byte(PCF8574)
    return val

# Initialize the devices
PCA9536_init()
MAX11600_init()

# Read from the configuration file
f = open("EmailNotify.conf","r")
# read title line
title = f.readline().split(",")
analog1 = f.readline().split(",")
analog2 = f.readline().split(",")
analog3 = f.readline().split(",")
analog4 = f.readline().split(",")
digital1 = f.readline().split(",")
digital2 = f.readline().split(",")
digital3 = f.readline().split(",")
digital4 = f.readline().split(",")
digital5 = f.readline().split(",")
digital6 = f.readline().split(",")
digital7 = f.readline().split(",")
digital8 = f.readline().split(",")
rate = f.readline()		# rate to read inputs
rate = rate[rate.find('=')+1:-1]
port = f.readline()		# smtp port
port = port[port.find('=')+1:-1]
password = f.readline()		# password
password = password[password.find('=')+1:-1]
smtpserver = f.readline()	# smtp server
smtpserver = smtpserver[smtpserver.find('=')+1:-1]
sender_email = f.readline()	# senders email
sender_email = sender_email[sender_email.find('=')+1:-1]
receiver_email = f.readline()	# receiver email
receiver_email = receiver_email[receiver_email.find('=')+1:-1]
f.close()

print 'Rec:' + rate + port + password + smtpserver + sender_email + receiver_email 
#list of lists
analog = [analog1,analog2,analog3,analog4]
digital = [digital1,digital2,digital3,digital4,digital5,digital6,digital7,digital8]

digitalErrorStatus = 0	#clear digital error flag
analogErrorStatus = 0 # clear analog error flag
# Main loop
while True:
    if (debug):
        print "analogErrorStatus: {0:b} ".format(analogErrorStatus)
        print "digitalErrorStatus: {0:b} ".format(digitalErrorStatus)
    ana1 = (MAX11600_read(MAX11600_CHANNEL0))
    ana2 = (MAX11600_read(MAX11600_CHANNEL1))
    ana3 = (MAX11600_read(MAX11600_CHANNEL2))
    ana4 = (MAX11600_read(MAX11600_CHANNEL3))
    val = [ana1,ana2,ana3,ana4]
    if (debug):
        print ("An1: % 4.2f" % ana1)
        print ("An2: % 4.2f" % ana2)
        print ("An3: % 4.2f" % ana3)
        print ("An4: % 4.2f" % ana4)
    # check each channel for an exception
    for chan in range(4):
        mask = 2**chan
        if analog[chan][ANALOG_enabled].upper() == "TRUE":
            # check if value outside of min/max    
	    if debug: print ('Chan: %i Value: %f Min: %s Max: %s') % (chan,val[chan],analog[chan][ANALOG_min],analog[chan][ANALOG_max])
            if ((val[chan] > float(analog[chan][ANALOG_max])) or (val[chan] < float(analog[chan][ANALOG_min])) and (analogErrorStatus & mask == 0)):
       	        analogErrorStatus |= mask
                if debug: print 'Sending set email'
                msg = "Analog %i: %s is at: %4.2f, outside of the min [%4.2f] or max [%4.2f] %s" % (chan,analog[chan][ANALOG_name],val[chan],float(analog[chan][ANALOG_min]),float(analog[chan][ANALOG_max]),analog[chan][ANALOG_units])
                SendMsg(smtpserver,port,msg,sender_email,receiver_email)
                #time.sleep(int(analog[chan][ANALOG_hysteresis]))
            # check if error is cleared 
            elif ((val[chan] < float(analog[chan][ANALOG_max])) and (val[chan] > float(analog[chan][ANALOG_min])) and ((analogErrorStatus & mask))):
       	        analogErrorStatus &= ~mask	# clear the error 
                if debug: print 'Sending clear email'
                msg = "Analog %i: %s is at: %4.2f, within limits, min [%4.2f] and max [%4.2f] %s" % (chan,analog[chan][ANALOG_name],val[chan],float(analog[chan][ANALOG_min]),float(analog[chan][ANALOG_max]),analog[chan][ANALOG_units])
                SendMsg(smtpserver,port,msg,sender_email,receiver_email)
                #time.sleep(int(analog[chan][ANALOG_hysteresis]))
    # check each digital channel for exception
    digitalInput = PCF8574_input()
    if (debug): print "Digital Input {0:b} ".format(digitalInput)
    for chan in range(8):
        mask = 2**chan
        if (digital[chan][DIGITAL_enabled]):
            dig = (digitalInput & mask) == mask
            # if we have an error and the error hasnt been reported
            if debug: print ('Digital: %i Value: %i Error: %i') % (chan,dig,int(digital[chan][DIGITAL_error]))
            if (dig == int(digital[chan][DIGITAL_error])) and ((digitalErrorStatus & mask) == 0):
                digitalErrorStatus |= mask   # set a error status bit so we can check if all clear     
                msg = 'Digital 1: %s is in Error state: %i' % (digital[chan][DIGITAL_name],dig)
                SendMsg(smtpserver,port,msg,sender_email,receiver_email)
                #time.sleep(int(digital[chan][DIGITAL_hysteresis]))
            # if we had an error but but now is clear 
            elif (dig != int(digital[chan][DIGITAL_error]) and (digitalErrorStatus & mask) == mask):
                digitalErrorStatus &= ~mask	# clear the error
                msg = 'Digital 1: %s error is Clear state: %i' % (digital[chan][DIGITAL_name],dig)
                SendMsg(smtpserver,port,msg,sender_email,receiver_email)
                #time.sleep(int(digital[chan][DIGITAL_hysteresis]))
    time.sleep(int(rate))
